@@ -4,7 +4,6 @@
 #include "amount.h"
 
 #include <map>
-#include <ctime>
 #include <iostream>
 
 namespace ledger {
@@ -35,13 +34,13 @@ class balance_t
       *this += (*i).second;
   }
   balance_t(const amount_t& amt) {
-    if (amt)
+    if (! amt.realzero())
       amounts.insert(amounts_pair(&amt.commodity(), amt));
   }
   template <typename T>
   balance_t(T value) {
     amount_t amt(value);
-    if (amt)
+    if (! amt.realzero())
       amounts.insert(amounts_pair(&amt.commodity(), amt));
   }
 
@@ -80,7 +79,7 @@ class balance_t
     amounts_map::iterator i = amounts.find(&amt.commodity());
     if (i != amounts.end())
       (*i).second += amt;
-    else if (amt)
+    else if (! amt.realzero())
       amounts.insert(amounts_pair(&amt.commodity(), amt));
     return *this;
   }
@@ -97,10 +96,14 @@ class balance_t
   }
   balance_t& operator-=(const amount_t& amt) {
     amounts_map::iterator i = amounts.find(&amt.commodity());
-    if (i != amounts.end())
+    if (i != amounts.end()) {
       (*i).second -= amt;
-    else if (amt)
-      amounts.insert(amounts_pair(&amt.commodity(), amt));
+      if ((*i).second.realzero())
+	amounts.erase(i);
+    }
+    else if (! amt.realzero()) {
+      amounts.insert(amounts_pair(&amt.commodity(), - amt));
+    }
     return *this;
   }
   template <typename T>
@@ -143,59 +146,15 @@ class balance_t
   }
 
   // multiplication and divide
-  balance_t& operator*=(const balance_t& bal) {
-    if (amounts.size() == 1 && bal.amounts.size() == 1)
-      return *this *= (*bal.amounts.begin()).second;
-    throw amount_error("It makes no sense to multiply two balances");
-  }
-  balance_t& operator*=(const amount_t& amt) {
-    // Multiplying by the null commodity causes all amounts to be
-    // increased by the same factor.
-    if (amt.commodity().symbol.empty()) {
-      for (amounts_map::iterator i = amounts.begin();
-	   i != amounts.end();
-	   i++)
-	(*i).second *= amt;
-    }
-    else if (amounts.size() == 1) {
-      (*amounts.begin()).second *= amt;
-    }
-    else {
-      amounts_map::iterator i = amounts.find(&amt.commodity());
-      if (i != amounts.end())
-	(*i).second *= amt;
-    }
-    return *this;
-  }
+  balance_t& operator*=(const balance_t& bal);
+  balance_t& operator*=(const amount_t& amt);
   template <typename T>
   balance_t& operator*=(T val) {
     return *this *= amount_t(val);
   }
 
-  balance_t& operator/=(const balance_t& bal) {
-    if (amounts.size() == 1 && bal.amounts.size() == 1)
-      return *this /= (*bal.amounts.begin()).second;
-    throw amount_error("It makes no sense to divide two balances");
-  }
-  balance_t& operator/=(const amount_t& amt) {
-    // Dividing by the null commodity causes all amounts to be
-    // increased by the same factor.
-    if (amt.commodity().symbol.empty()) {
-      for (amounts_map::iterator i = amounts.begin();
-	   i != amounts.end();
-	   i++)
-	(*i).second /= amt;
-    }
-    else if (amounts.size() == 1) {
-      (*amounts.begin()).second /= amt;
-    }
-    else {
-      amounts_map::iterator i = amounts.find(&amt.commodity());
-      if (i != amounts.end())
-	(*i).second /= amt;
-    }
-    return *this;
-  }
+  balance_t& operator/=(const balance_t& bal);
+  balance_t& operator/=(const amount_t& amt);
   template <typename T>
   balance_t& operator/=(T val) {
     return *this /= amount_t(val);
@@ -255,7 +214,15 @@ class balance_t
     return true;
   }
   bool operator<(const amount_t& amt) const {
-    return amount(amt.commodity()) < amt;
+    if (amt.commodity())
+      return amount(amt.commodity()) < amt;
+
+    for (amounts_map::const_iterator i = amounts.begin();
+	 i != amounts.end();
+	 i++)
+      if ((*i).second < amt)
+	return true;
+    return false;
   }
   template <typename T>
   bool operator<(T val) const {
@@ -283,7 +250,15 @@ class balance_t
     return true;
   }
   bool operator<=(const amount_t& amt) const {
-    return amount(amt.commodity()) <= amt;
+    if (amt.commodity())
+      return amount(amt.commodity()) <= amt;
+
+    for (amounts_map::const_iterator i = amounts.begin();
+	 i != amounts.end();
+	 i++)
+      if ((*i).second <= amt)
+	return true;
+    return false;
   }
   template <typename T>
   bool operator<=(T val) const {
@@ -314,7 +289,15 @@ class balance_t
     return true;
   }
   bool operator>(const amount_t& amt) const {
-    return amount(amt.commodity()) > amt;
+    if (amt.commodity())
+      return amount(amt.commodity()) > amt;
+
+    for (amounts_map::const_iterator i = amounts.begin();
+	 i != amounts.end();
+	 i++)
+      if ((*i).second > amt)
+	return true;
+    return false;
   }
   template <typename T>
   bool operator>(T val) const {
@@ -342,7 +325,15 @@ class balance_t
     return true;
   }
   bool operator>=(const amount_t& amt) const {
-    return amount(amt.commodity()) >= amt;
+    if (amt.commodity())
+      return amount(amt.commodity()) >= amt;
+
+    for (amounts_map::const_iterator i = amounts.begin();
+	 i != amounts.end();
+	 i++)
+      if ((*i).second >= amt)
+	return true;
+    return false;
   }
   template <typename T>
   bool operator>=(T val) const {
@@ -366,7 +357,15 @@ class balance_t
     return i == amounts.end() && j == bal.amounts.end();
   }
   bool operator==(const amount_t& amt) const {
-    return amounts.size() == 1 && (*amounts.begin()).second == amt;
+    if (amt.commodity())
+      return amounts.size() == 1 && (*amounts.begin()).second == amt;
+
+    for (amounts_map::const_iterator i = amounts.begin();
+	 i != amounts.end();
+	 i++)
+      if ((*i).second == amt)
+	return true;
+    return false;
   }
   template <typename T>
   bool operator==(T val) const {
@@ -406,15 +405,7 @@ class balance_t
   }
 
   // conversion operators
-  operator amount_t() const {
-    if (amounts.size() == 1)
-      return (*amounts.begin()).second;
-    else if (amounts.size() == 0)
-      return amount_t();
-    else
-      throw amount_error("Cannot convert a balance with "
-			 "multiple commodities to an amount");
-  }
+  operator amount_t() const;
   operator bool() const {
     for (amounts_map::const_iterator i = amounts.begin();
 	 i != amounts.end();
@@ -424,12 +415,30 @@ class balance_t
     return false;
   }
 
-  amount_t  amount(const commodity_t& commodity) const;
-  balance_t value(const std::time_t moment) const;
+  bool realzero() const {
+    if (amounts.size() == 0)
+      return true;
+    for (amounts_map::const_iterator i = amounts.begin();
+	 i != amounts.end();
+	 i++)
+      if (! (*i).second.realzero())
+	return false;
+    return true;
+  }
 
-  void      write(std::ostream& out,
-		  const int	first_width,
-		  const int	latter_width = -1) const;
+  amount_t   amount(const commodity_t& commodity =
+		    *commodity_t::null_commodity) const;
+  balance_t  value(const datetime_t& moment = datetime_t::now) const;
+  balance_t  price() const;
+  datetime_t date() const;
+
+  balance_t
+  strip_annotations(const bool keep_price = amount_t::keep_price,
+		    const bool keep_date  = amount_t::keep_date,
+		    const bool keep_tag   = amount_t::keep_tag) const;
+
+  void write(std::ostream& out, const int first_width,
+	     const int latter_width = -1) const;
 
   void abs() {
     for (amounts_map::iterator i = amounts.begin();
@@ -438,12 +447,35 @@ class balance_t
       (*i).second.abs();
   }
 
+  void reduce() {
+    for (amounts_map::iterator i = amounts.begin();
+	 i != amounts.end();
+	 i++)
+      (*i).second.reduce();
+  }
+
+  balance_t reduced() const {
+    balance_t temp(*this);
+    temp.reduce();
+    return temp;
+  }
+
   void round() {
     for (amounts_map::iterator i = amounts.begin();
 	 i != amounts.end();
 	 i++)
       if ((*i).second.commodity())
-	(*i).second = (*i).second.round((*i).second.commodity().precision);
+	(*i).second = (*i).second.round();
+  }
+
+  balance_t unround() const {
+    balance_t temp;
+    for (amounts_map::const_iterator i = amounts.begin();
+	 i != amounts.end();
+	 i++)
+      if ((*i).second.commodity())
+	temp += (*i).second.unround();
+    return temp;
   }
 };
 
@@ -458,11 +490,10 @@ inline std::ostream& operator<<(std::ostream& out, const balance_t& bal) {
   return out;
 }
 
-
 class balance_pair_t
 {
  public:
-  balance_t quantity;
+  balance_t   quantity;
   balance_t * cost;
 
   // constructors
@@ -481,8 +512,7 @@ class balance_pair_t
 
   // destructor
   ~balance_pair_t() {
-    if (cost)
-      delete cost;
+    if (cost) delete cost;
   }
 
   // assignment operator
@@ -492,7 +522,6 @@ class balance_pair_t
 	delete cost;
 	cost = NULL;
       }
-
       quantity = bal_pair.quantity;
       if (bal_pair.cost)
 	cost = new balance_t(*bal_pair.cost);
@@ -529,12 +558,9 @@ class balance_pair_t
   balance_pair_t& operator+=(const balance_pair_t& bal_pair) {
     if (bal_pair.cost && ! cost)
       cost = new balance_t(quantity);
-
     quantity += bal_pair.quantity;
-
     if (cost)
       *cost += bal_pair.cost ? *bal_pair.cost : bal_pair.quantity;
-
     return *this;
   }
   balance_pair_t& operator+=(const balance_t& bal) {
@@ -557,12 +583,9 @@ class balance_pair_t
   balance_pair_t& operator-=(const balance_pair_t& bal_pair) {
     if (bal_pair.cost && ! cost)
       cost = new balance_t(quantity);
-
     quantity -= bal_pair.quantity;
-
     if (cost)
       *cost -= bal_pair.cost ? *bal_pair.cost : bal_pair.quantity;
-
     return *this;
   }
   balance_pair_t& operator-=(const balance_t& bal) {
@@ -631,11 +654,9 @@ class balance_pair_t
   balance_pair_t& operator*=(const balance_pair_t& bal_pair) {
     if (bal_pair.cost && ! cost)
       cost = new balance_t(quantity);
-
     quantity *= bal_pair.quantity;
     if (cost)
       *cost *= bal_pair.cost ? *bal_pair.cost : bal_pair.quantity;
-
     return *this;
   }
   balance_pair_t& operator*=(const balance_t& bal) {
@@ -658,11 +679,9 @@ class balance_pair_t
   balance_pair_t& operator/=(const balance_pair_t& bal_pair) {
     if (bal_pair.cost && ! cost)
       cost = new balance_t(quantity);
-
     quantity /= bal_pair.quantity;
     if (cost)
       *cost /= bal_pair.cost ? *bal_pair.cost : bal_pair.quantity;
-
     return *this;
   }
   balance_pair_t& operator/=(const balance_t& bal) {
@@ -814,8 +833,7 @@ class balance_pair_t
   // unary negation
   void negate() {
     quantity.negate();
-    if (cost)
-      cost->negate();
+    if (cost) cost->negate();
   }
   balance_pair_t negated() const {
     balance_pair_t temp = *this;
@@ -827,31 +845,48 @@ class balance_pair_t
   }
 
   // test for non-zero (use ! for zero)
-  operator bool() const {
-    return quantity;
-  }
   operator balance_t() const {
     return quantity;
   }
   operator amount_t() const {
     return quantity;
   }
+  operator bool() const {
+    return quantity;
+  }
+
+  bool realzero() const {
+    return ((! cost || cost->realzero()) && quantity.realzero());
+  }
 
   void abs() {
     quantity.abs();
-    if (cost)
-      cost->abs();
+    if (cost) cost->abs();
   }
 
-  amount_t  amount(const commodity_t& commodity) const {
+  amount_t  amount(const commodity_t& commodity =
+		   *commodity_t::null_commodity) const {
     return quantity.amount(commodity);
   }
-  balance_t value(const std::time_t moment) const {
+  balance_t value(const datetime_t& moment = datetime_t::now) const {
     return quantity.value(moment);
   }
-  void      write(std::ostream& out,
-		  const int	first_width,
-		  const int	latter_width = -1) const {
+  balance_t price() const {
+    return quantity.price();
+  }
+  datetime_t date() const {
+    return quantity.date();
+  }
+
+  balance_t
+  strip_annotations(const bool keep_price = amount_t::keep_price,
+		    const bool keep_date  = amount_t::keep_date,
+		    const bool keep_tag   = amount_t::keep_tag) const {
+    return quantity.strip_annotations(keep_price, keep_date, keep_tag);
+  }
+
+  void write(std::ostream& out, const int first_width,
+	     const int latter_width = -1) const {
     quantity.write(out, first_width, latter_width);
   }
 
@@ -859,11 +894,9 @@ class balance_pair_t
 		      const amount_t * a_cost = NULL) {
     if (a_cost && ! cost)
       cost = new balance_t(quantity);
-
     quantity += amount;
     if (cost)
       *cost += a_cost ? *a_cost : amount;
-
     return *this;
   }
 
@@ -871,8 +904,43 @@ class balance_pair_t
     return quantity.valid() && (! cost || cost->valid());
   }
 
+  void reduce() {
+    quantity.reduce();
+    if (cost) cost->reduce();
+  }
+
+  balance_pair_t reduced() const {
+    balance_pair_t temp(*this);
+    temp.reduce();
+    return temp;
+  }
+
   void round() {
     quantity.round();
+    if (cost) cost->round();
+  }
+
+  balance_pair_t unround() {
+    balance_pair_t temp(quantity.unround());
+    if (cost)
+      temp.cost = new balance_t(cost->unround());
+    return temp;
+  }
+
+  friend inline bool operator<(const balance_t& bal, const balance_pair_t& bal_pair) {
+    return bal_pair >= bal;
+  }
+  friend inline bool operator<=(const balance_t& bal, const balance_pair_t& bal_pair) {
+    return bal_pair > bal;
+  }
+  friend inline bool operator>(const balance_t& bal, const balance_pair_t& bal_pair) {
+    return bal_pair <= bal;
+  }
+  friend inline bool operator>=(const balance_t& bal, const balance_pair_t& bal_pair) {
+    return bal_pair < bal;
+  }
+  friend inline bool operator==(const balance_t& bal, const balance_pair_t& bal_pair) {
+    return bal_pair == bal;
   }
 };
 
