@@ -20,6 +20,9 @@ class timing_t
   timing_t(const std::string& _symbol, const std::string& _category)
     : begin(0), cumulative(0), symbol(_symbol), category(_category) {}
 
+  timing_t(const std::string& _symbol)
+    : begin(0), cumulative(0), symbol(_symbol) {}
+
   ~timing_t() {
     std::string cls = "timing.results.";
     cls += symbol;
@@ -33,6 +36,9 @@ class timing_t
     line  = _line;
     begin = std::clock();
   }
+  void start() {
+    begin = std::clock();
+  }
 
   void stop() {
     cumulative += std::clock() - begin;
@@ -41,10 +47,12 @@ class timing_t
 
 #ifdef DEBUG_ENABLED
 #define TIMER_DEF(sym, cat) static timing_t sym(#sym, cat)
+#define TIMER_DEF_(sym) static timing_t sym(#sym, #sym)
 #define TIMER_START(sym) sym.start(__FILE__, __LINE__)
 #define TIMER_STOP(sym) sym.stop()
 #else
 #define TIMER_DEF(sym, cat)
+#define TIMER_DEF_(sym)
 #define TIMER_START(sym)
 #define TIMER_STOP(sym)
 #endif
